@@ -1,8 +1,7 @@
 import abc
 from uuid import UUID
 
-from libecalc.common.utils.rates import TimeSeriesRate
-from libecalc.core.result.emission import EmissionResult
+from libecalc.common.utils.rates import TimeSeriesRate, TimeSeriesStreamDayRate
 from libecalc.domain.energy.component_energy_context import ComponentEnergyContext
 from libecalc.domain.energy.energy_model import EnergyModel
 
@@ -26,7 +25,7 @@ class Emitter(abc.ABC):
         self,
         energy_context: ComponentEnergyContext,
         energy_model: EnergyModel,
-    ) -> dict[str, EmissionResult] | None: ...
+    ) -> dict[str, TimeSeriesStreamDayRate] | None: ...
 
     @abc.abstractmethod
     def get_emissions(self) -> dict[EmissionName, TimeSeriesRate]: ...
