@@ -12,6 +12,7 @@ from libecalc.domain.infrastructure.energy_components.turbine.turbine import Tur
 from libecalc.domain.process.compressor.core.train.utils.common import POWER_CALCULATION_TOLERANCE
 from libecalc.domain.process.compressor.core.train.utils.numeric_methods import find_root
 from libecalc.domain.process.core.results import CompressorTrainResult
+from libecalc.domain.process.value_objects.fluid_stream.fluid_factory import FluidFactoryInterface
 
 
 class CompressorModel:
@@ -21,6 +22,7 @@ class CompressorModel:
     def set_evaluation_input(
         self,
         rate: NDArray[np.float64],
+        fluid_factory: FluidFactoryInterface | list[FluidFactoryInterface] | None,
         suction_pressure: NDArray[np.float64] | None,
         discharge_pressure: NDArray[np.float64] | None,
         intermediate_pressure: NDArray[np.float64] | None = None,
@@ -29,6 +31,7 @@ class CompressorModel:
 
         Args:
             rate (NDArray[np.float64]): Actual volumetric rate in [Sm3/h].
+            fluid_factory (FluidFactoryInterface | list[FluidFactoryInterface] | None): Fluid
             suction_pressure (NDArray[np.float64]): Suction pressure per time step in [bara].
             discharge_pressure (NDArray[np.float64]): Discharge pressure per time step in [bara].
             intermediate_pressure (NDArray[np.float64] | None): Intermediate pressure per time step in [bara], or None.
@@ -46,12 +49,14 @@ class CompressorModel:
         self,
         suction_pressures: NDArray[np.float64],
         discharge_pressures: NDArray[np.float64],
+        fluid_factory: FluidFactoryInterface | None = None,
     ) -> NDArray[np.float64]:
         """Get the maximum standard flow rate [Sm3/day] for the compressor train. This method is valid for compressor
         trains where there are a single input stream and no streams are added or removed in the train.
 
         :param suction_pressures: Suction pressure per time step [bara]
         :param discharge_pressures: Discharge pressure per time step [bara]
+        :param fluid_factory: Fluid factory interface
         :return: Maximum standard rate per day per time-step [Sm3/day]
         """
         raise NotImplementedError
@@ -128,38 +133,49 @@ class CompressorWithTurbineModel(CompressorModel):
         self, standard_rate: float, suction_pressure: float, discharge_pressure: float, max_power: float
     ) -> float:
         """Expression used in optimization to find the rate that utilizes the compressor trains capacity."""
+
         self.compressor_model.set_evaluation_input(
+            fluid_factory=self.compressor_model._fluid_factory,
             rate=np.asarray([standard_rate]),
             suction_pressure=np.asarray([suction_pressure]),
             discharge_pressure=np.asarray([discharge_pressure]),
         )
         result = self.compressor_model.evaluate()
-        if result.power is None or len(result.power) == 0:
+        energy_result = result.get_energy_result()
+        if energy_result.power is None or len(energy_result.power.values) == 0:
             return 0.0  # Return 0 if no power value available
-        return float(result.power[0]) - (max_power - POWER_CALCULATION_TOLERANCE)
+        return float(energy_result.power.values[0]) - (max_power - POWER_CALCULATION_TOLERANCE)
 
     def get_max_standard_rate(
-        self, suction_pressures: NDArray[np.float64], discharge_pressures: NDArray[np.float64]
+        self,
+        suction_pressures: NDArray[np.float64],
+        discharge_pressures: NDArray[np.float64],
+        fluid_factory: FluidFactoryInterface | None = None,
     ) -> NDArray[np.float64]:
         """Validate that the compressor has enough power to handle the set maximum standard rate.
         If there is insufficient power find new maximum rate.
         """
+        if fluid_factory is not None:
+            self.compressor_model._fluid_factory = fluid_factory
+
         max_standard_rate = self.compressor_model.get_max_standard_rate(
             suction_pressures=suction_pressures, discharge_pressures=discharge_pressures
         )
 
         # Check if the obtained results are within the maximum load that the turbine can deliver
         self.compressor_model.set_evaluation_input(
+            fluid_factory=self.compressor_model._fluid_factory,
             rate=max_standard_rate,
             suction_pressure=suction_pressures,
             discharge_pressure=discharge_pressures,
         )
         results_max_standard_rate = self.compressor_model.evaluate()
+        energy_result = results_max_standard_rate.get_energy_result()
 
         max_power = self.turbine_model.max_power
 
-        if results_max_standard_rate.power is not None:
-            powers = np.asarray(results_max_standard_rate.power)
+        if energy_result.power is not None:
+            powers = np.asarray(energy_result.power.values)
             for i, (power, suction_pressure, discharge_pressure) in enumerate(
                 zip(powers, suction_pressures, discharge_pressures)
             ):
