@@ -17,23 +17,26 @@ from libecalc.domain.process.compressor.core.results import (
 from libecalc.domain.process.compressor.core.train.stage import CompressorTrainStage
 from libecalc.domain.process.compressor.core.train.train_evaluation_input import CompressorTrainEvaluationInput
 from libecalc.domain.process.compressor.core.train.utils.common import EPSILON, PRESSURE_CALCULATION_TOLERANCE
-from libecalc.domain.process.compressor.core.train.utils.numeric_methods import (
-    find_root,
-    maximize_x_given_boolean_condition_function,
-)
 from libecalc.domain.process.core.results import CompressorTrainResult
 from libecalc.domain.process.core.results.compressor import TargetPressureStatus
+from libecalc.domain.process.value_objects.fluid_stream import FluidStream
 from libecalc.domain.process.value_objects.fluid_stream.fluid_factory import FluidFactoryInterface
 
 INVALID_MAX_RATE = np.nan
 
 
+def calculate_pressure_ratio_per_stage(suction_pressure: float, discharge_pressure: float, n_stages: int):
+    if n_stages < 1:
+        raise ValueError("Can't compute pressure rations when no compressor stages are defined.")
+    pressure_ratios = discharge_pressure / suction_pressure
+    return pressure_ratios ** (1.0 / n_stages)
+
+
 class CompressorTrainModel(CompressorModel, ABC):
-    """Base model for compressor trains with common shaft."""
+    """Base model for compressor trains."""
 
     def __init__(
         self,
-        fluid_factory: FluidFactoryInterface,
         energy_usage_adjustment_constant: float,
         energy_usage_adjustment_factor: float,
         stages: list[CompressorTrainStage],
@@ -46,7 +49,6 @@ class CompressorTrainModel(CompressorModel, ABC):
         # self.data_transfer_object = data_transfer_object
         self.energy_usage_adjustment_constant = energy_usage_adjustment_constant
         self.energy_usage_adjustment_factor = energy_usage_adjustment_factor
-        self.fluid_factory = fluid_factory
         self.stages = stages
         self.maximum_power = maximum_power
         self._maximum_discharge_pressure = maximum_discharge_pressure
@@ -61,12 +63,12 @@ class CompressorTrainModel(CompressorModel, ABC):
     @property
     def minimum_speed(self) -> float:
         """Determine the minimum speed of the compressor train if variable speed. Otherwise, it doesn't make sense."""
-        return max([stage.compressor_chart.minimum_speed for stage in self.stages])
+        return max([stage.compressor.compressor_chart.minimum_speed for stage in self.stages])
 
     @property
     def maximum_speed(self) -> float:
         """Determine the maximum speed of the compressor train if variable speed. Otherwise it doesn't make sense."""
-        return min([stage.compressor_chart.maximum_speed for stage in self.stages])
+        return min([stage.compressor.compressor_chart.maximum_speed for stage in self.stages])
 
     @property
     def pressure_control(self) -> FixedSpeedPressureControl | None:
@@ -89,6 +91,7 @@ class CompressorTrainModel(CompressorModel, ABC):
     def set_evaluation_input(
         self,
         rate: NDArray[np.float64],
+        fluid_factory: FluidFactoryInterface | list[FluidFactoryInterface] | None,
         suction_pressure: NDArray[np.float64] | None,
         discharge_pressure: NDArray[np.float64] | None,
         intermediate_pressure: NDArray[np.float64] | None = None,
@@ -102,6 +105,7 @@ class CompressorTrainModel(CompressorModel, ABC):
         self._suction_pressure = suction_pressure
         self._discharge_pressure = discharge_pressure
         self._intermediate_pressure = intermediate_pressure
+        self._fluid_factory = fluid_factory
 
     def evaluate(
         self,
@@ -135,7 +139,7 @@ class CompressorTrainModel(CompressorModel, ABC):
             "and potential inter-stage pressure."
         )
 
-        train_results = []
+        train_results: list[CompressorTrainResultSingleTimeStep] = []
         for rate_value, suction_pressure_value, intermediate_pressure_value, discharge_pressure_value in zip(
             np.transpose(self._rate),
             self._suction_pressure,
@@ -180,7 +184,7 @@ class CompressorTrainModel(CompressorModel, ABC):
             stage_results,
         ) = CompressorTrainResultSingleTimeStep.from_result_list_to_dto(
             result_list=train_results,
-            compressor_charts=[stage.compressor_chart.data_transfer_object for stage in self.stages],
+            compressor_charts=[stage.compressor.compressor_chart for stage in self.stages],
         )
 
         return CompressorTrainResult(
@@ -219,24 +223,37 @@ class CompressorTrainModel(CompressorModel, ABC):
         """
         ...
 
+    def train_inlet_stream(
+        self,
+        pressure: float,
+        temperature: float,
+        rate: float,
+    ) -> FluidStream:
+        """Find inlet stream given constraints.
+
+        Args:
+            constraints (CompressorTrainEvaluationInput): The constraints for the evaluation.
+
+        Returns:
+            FluidStream: Inlet fluid stream at the compressor train inlet.
+        """
+        return self._fluid_factory.create_stream_from_standard_rate(
+            pressure_bara=pressure,
+            temperature_kelvin=temperature,
+            standard_rate_m3_per_day=rate,
+        )
+
     def calculate_pressure_ratios_per_stage(
         self,
-        suction_pressure: NDArray[np.float64] | float,
-        discharge_pressure: NDArray[np.float64] | float,
-    ) -> NDArray[np.float64] | float:
+        suction_pressure: float,
+        discharge_pressure: float,
+    ) -> float:
         """Given the number of compressors, and based on the assumption that all compressors have the same pressure ratio,
         compute all pressure ratios.
         """
-        if len(self.stages) < 1:
-            raise ValueError("Can't compute pressure rations when no compressor stages are defined.")
-        if isinstance(suction_pressure, np.ndarray):
-            pressure_ratios = np.divide(
-                discharge_pressure, suction_pressure, out=np.ones_like(suction_pressure), where=suction_pressure != 0
-            )
-            return pressure_ratios ** (1.0 / len(self.stages))
-        else:
-            pressure_ratios = discharge_pressure / suction_pressure
-            return pressure_ratios ** (1.0 / len(self.stages))
+        return calculate_pressure_ratio_per_stage(
+            suction_pressure=suction_pressure, discharge_pressure=discharge_pressure, n_stages=len(self.stages)
+        )
 
     def check_target_pressures(
         self,
@@ -292,6 +309,7 @@ class CompressorTrainModel(CompressorModel, ABC):
         self,
         suction_pressures: NDArray[np.float64],
         discharge_pressures: NDArray[np.float64],
+        fluid_factory: FluidFactoryInterface | None = None,
     ) -> NDArray[np.float64]:
         """
         Calculate the maximum standard volume rate [Sm3/day] that the compressor train can operate at.
@@ -303,11 +321,14 @@ class CompressorTrainModel(CompressorModel, ABC):
         Args:
             suction_pressures (float): The suction pressures in bara for each time step.
             discharge_pressures (float): The discharge pressures in bara for each time step.
+            fluid_factory (FluidFactoryInterface): The fluid factory interface.
 
         Returns:
             NDArray[np.float64]: An array of maximum standard rates for each time step.
             If the maximum rate cannot be determined, it returns INVALID_MAX_RATE for that time step.
         """
+        if fluid_factory is not None:
+            self._fluid_factory = fluid_factory
 
         max_standard_rate = np.full_like(suction_pressures, fill_value=INVALID_MAX_RATE, dtype=float)
         for i, (suction_pressure_value, discharge_pressure_value) in enumerate(
@@ -322,103 +343,9 @@ class CompressorTrainModel(CompressorModel, ABC):
                 rate=EPSILON,
             )
             try:
-                max_standard_rate[i] = self._get_max_std_rate_single_timestep(
-                    constraints=constraints,
-                )
+                max_standard_rate[i] = self._get_max_std_rate_single_timestep(constraints=constraints)
             except EcalcError as e:
                 logger.exception(e)
                 max_standard_rate[i] = float("nan")
 
         return max_standard_rate
-
-    def find_shaft_speed_given_constraints(
-        self,
-        constraints: CompressorTrainEvaluationInput,
-        lower_bound_for_speed: float | None = None,
-        upper_bound_for_speed: float | None = None,
-    ) -> float:
-        """Calculate needed shaft speed to get desired outlet pressure
-
-        Run compressor train forward model with inlet conditions and speed, and iterate on shaft speed until discharge
-        pressure meets requested discharge pressure.
-
-        Iteration (using brenth method) to find speed to meet requested discharge pressure
-
-        The upper and lower bounds for the speed can be set, which is useful for a part of a compressor train that can
-        share a common shaft with another part of a compressor train, which has another minimum and maximum speed.
-
-        Iterative problem:
-            f(speed) = calculate_compressor_train(speed).discharge_pressure - requested_discharge_pressure = 0
-        Starting points for iterative method:
-           speed_0 = minimum speed for train, calculate f(speed_0) aka f_0
-           speed_1 = maximum speed for train, calculate f(speed_1) aka f_1
-
-        Args:
-            constraints (CompressorTrainEvaluationInput): The constraints for the evaluation.
-            lower_bound_for_speed (float | None): The lower bound for the speed. If None, uses the minimum speed
-            upper_bound_for_speed (float | None): The upper bound for the speed. If None, uses the maximum speed
-        Returns:
-            The speed required to operate at to meet the given constraints. (Bounded by the minimu and maximum speed)
-
-        """
-        minimum_speed = (
-            lower_bound_for_speed
-            if lower_bound_for_speed and lower_bound_for_speed > self.minimum_speed
-            else self.minimum_speed
-        )
-        maximum_speed = (
-            upper_bound_for_speed
-            if upper_bound_for_speed and upper_bound_for_speed < self.maximum_speed
-            else self.maximum_speed
-        )
-        if constraints.speed is not None:
-            return constraints.speed
-
-        def _calculate_compressor_train(_speed: float) -> CompressorTrainResultSingleTimeStep:
-            return self.calculate_compressor_train(
-                constraints=constraints.create_conditions_with_new_input(
-                    new_speed=_speed,
-                )
-            )
-
-        train_result_for_minimum_speed = _calculate_compressor_train(_speed=minimum_speed)
-        train_result_for_maximum_speed = _calculate_compressor_train(_speed=maximum_speed)
-
-        if not train_result_for_maximum_speed.within_capacity:
-            # will not find valid result - the rate is above maximum rate, return invalid results at maximum speed
-            return maximum_speed
-        if not train_result_for_minimum_speed.within_capacity:
-            # rate is above maximum rate for minimum speed. Find the lowest minimum speed which gives a valid result
-            minimum_speed = -maximize_x_given_boolean_condition_function(
-                x_min=-maximum_speed,
-                x_max=-minimum_speed,
-                bool_func=lambda x: _calculate_compressor_train(_speed=-x).within_capacity,
-            )
-            train_result_for_minimum_speed = _calculate_compressor_train(_speed=minimum_speed)
-
-        # Solution 1, iterate on speed until target discharge pressure is found
-        if (
-            constraints.discharge_pressure is not None
-            and train_result_for_minimum_speed.discharge_pressure
-            <= constraints.discharge_pressure
-            <= train_result_for_maximum_speed.discharge_pressure
-        ):
-            # At this point, discharge_pressure is confirmed to be not None
-            target_discharge_pressure = constraints.discharge_pressure
-            speed = find_root(
-                lower_bound=minimum_speed,
-                upper_bound=maximum_speed,
-                func=lambda x: _calculate_compressor_train(_speed=x).discharge_pressure - target_discharge_pressure,
-            )
-
-            return speed
-
-        # Solution 2, target pressure is too low:
-        if (
-            constraints.discharge_pressure is not None
-            and constraints.discharge_pressure < train_result_for_minimum_speed.discharge_pressure
-        ):
-            return minimum_speed
-
-        # Solution 3, target discharge pressure is too high
-        return maximum_speed
