@@ -25,9 +25,9 @@ from libecalc.domain.process.compressor.core.train.utils.numeric_methods import
     maximize_x_given_boolean_condition_function,
 )
 from libecalc.domain.process.core.results.compressor import TargetPressureStatus
+from libecalc.domain.process.entities.shaft import Shaft, SingleSpeedShaft, VariableSpeedShaft
 from libecalc.domain.process.value_objects.chart.chart_area_flag import ChartAreaFlag
 from libecalc.domain.process.value_objects.fluid_stream import ProcessConditions
-from libecalc.domain.process.value_objects.fluid_stream.fluid_factory import FluidFactoryInterface
 
 
 class CompressorTrainCommonShaft(CompressorTrainModel):
@@ -43,6 +43,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         energy_usage_adjustment_constant (float): Constant to be added to the computed power. Defaults to 0.0.
         energy_usage_adjustment_factor (float): Factor to be multiplied to computed power. Defaults to 1.0.
         stages (list[CompressorTrainStage]): List of compressor stages in the train.
+        shaft (Shaft): The shaft the compressor stages are mounted on.
         pressure_control (FixedSpeedPressureControl | None, optional): If set, the compressor train will
             operate with pressure control. Meaning that if the target discharge pressure is not reached at the given
             speed, the compressor train will operate at the maximum possible discharge pressure at the given speed.
@@ -63,10 +64,10 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
 
     def __init__(
         self,
-        fluid_factory: FluidFactoryInterface,
         energy_usage_adjustment_constant: float,
         energy_usage_adjustment_factor: float,
         stages: list[CompressorTrainStage],
+        shaft: Shaft,
         pressure_control: FixedSpeedPressureControl | None = None,
         calculate_max_rate: bool = False,
         maximum_power: float | None = None,
@@ -74,8 +75,8 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         stage_number_interstage_pressure: int | None = None,
     ):
         logger.debug(f"Creating CompressorTrainCommonShaft with n_stages: {len(stages)}")
+        self.shaft = shaft
         super().__init__(
-            fluid_factory=fluid_factory,
             energy_usage_adjustment_constant=energy_usage_adjustment_constant,
             energy_usage_adjustment_factor=energy_usage_adjustment_factor,
             stages=stages,
@@ -87,27 +88,24 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         )
         self._validate_maximum_discharge_pressure()
         self._validate_stages(stages)
+        self._validate_shaft()
+
+    @property
+    def is_variable_speed(self):
+        return all(stage.compressor.compressor_chart.is_variable_speed for stage in self.stages)
 
     def evaluate_given_constraints(
         self,
         constraints: CompressorTrainEvaluationInput,
+        fixed_speed: float | None = None,
     ) -> CompressorTrainResultSingleTimeStep:
         if constraints.rate > 0:  # type: ignore[operator]
-            if constraints.speed is None:
-                speed = self.find_shaft_speed_given_constraints(
-                    constraints=constraints,
-                )
-                train_result = self.calculate_compressor_train(
-                    constraints=constraints.create_conditions_with_new_input(
-                        new_speed=speed,
-                    ),
-                )
-            else:
-                speed = constraints.speed
-                train_result = self.calculate_compressor_train(
-                    constraints=constraints,
-                )
-
+            if fixed_speed is None:
+                fixed_speed = self.find_fixed_shaft_speed_given_constraints(constraints=constraints)
+            self.shaft.set_speed(fixed_speed)
+            train_result = self.calculate_compressor_train(
+                constraints=constraints,
+            )
             if train_result.target_pressure_status == TargetPressureStatus.TARGET_PRESSURES_MET:
                 # Solution found
                 return train_result
@@ -118,11 +116,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             elif self.pressure_control is None:
                 return train_result
             else:
-                train_result = self.evaluate_with_pressure_control_given_constraints(
-                    constraints=constraints.create_conditions_with_new_input(
-                        new_speed=speed,
-                    )
-                )
+                train_result = self.evaluate_with_pressure_control_given_constraints(constraints=constraints)
             return train_result
         else:
             return CompressorTrainResultSingleTimeStep.create_empty(number_of_stages=len(self.stages))
@@ -144,17 +138,17 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             results including conditions and calculations for each stage and power.
 
         """
-        if constraints.speed is None or constraints.rate is None or constraints.suction_pressure is None:
+        if not self.shaft.speed_is_defined or constraints.rate is None or constraints.suction_pressure is None:
             raise EcalcError(
                 title="Missing required parameters",
                 message="Compressor train calculation requires speed, rate and suction pressure to be set.",
             )
         # Initialize stream at inlet of first compressor stage using fluid properties and inlet conditions
 
-        train_inlet_stream = self.fluid_factory.create_stream_from_standard_rate(
-            pressure_bara=constraints.suction_pressure,
-            temperature_kelvin=self.stages[0].inlet_temperature_kelvin,
-            standard_rate_m3_per_day=constraints.rate,
+        train_inlet_stream = self.train_inlet_stream(
+            pressure=constraints.suction_pressure,
+            temperature=self.stages[0].inlet_temperature_kelvin,
+            rate=constraints.rate,
         )
 
         stage_results: list[CompressorTrainStageResultSingleTimeStep] = []
@@ -163,7 +157,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             inlet_stream = outlet_stream
             stage_result = stage.evaluate(
                 inlet_stream_stage=inlet_stream,
-                speed=constraints.speed,
+                speed=self.shaft.get_speed(),
                 asv_rate_fraction=asv_rate_fraction,
                 asv_additional_mass_rate=asv_additional_mass_rate,
             )
@@ -181,7 +175,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             inlet_stream=train_inlet_stream,
             outlet_stream=outlet_stream,
             stage_results=stage_results,
-            speed=constraints.speed,
+            speed=self.shaft.get_speed(),
             above_maximum_power=sum([stage_result.power_megawatt for stage_result in stage_results])
             > self.maximum_power
             if self.maximum_power
@@ -189,13 +183,13 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             target_pressure_status=target_pressure_status,
         )
 
-    def _validate_stages(self, stages):
+    def _validate_stages(self, stages: list[CompressorTrainStage]):
         # Check that the compressor stages have overlapping speed ranges
         min_speed_per_stage = []
         max_speed_per_stage = []
         for stage in stages:
-            max_speed_per_stage.append(stage.compressor_chart.maximum_speed)
-            min_speed_per_stage.append(stage.compressor_chart.minimum_speed)
+            max_speed_per_stage.append(stage.compressor.compressor_chart.maximum_speed)
+            min_speed_per_stage.append(stage.compressor.compressor_chart.minimum_speed)
         if max(min_speed_per_stage) > min(max_speed_per_stage):
             msg = "Variable speed compressors in compressor train have incompatible compressor charts."
             f" Stage {min_speed_per_stage.index(max(min_speed_per_stage)) + 1}'s minimum speed is higher"
@@ -246,7 +240,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             Standard volume rate [Sm3/day]
 
         """
-        inlet_density = self.fluid_factory.create_thermo_system(
+        inlet_density = self._fluid_factory.create_thermo_system(
             pressure_bara=constraints.suction_pressure,  # type: ignore[arg-type]
             temperature_kelvin=self.stages[0].inlet_temperature_kelvin,
         ).density
@@ -255,10 +249,10 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             """Partial function of self.calculate_compressor_train_given_speed
             where we only pass mass_rate.
             """
+            self.shaft.set_speed(speed)
             return self.calculate_compressor_train(
                 constraints=constraints.create_conditions_with_new_input(
-                    new_rate=self.fluid_factory.mass_rate_to_standard_rate(mass_rate),  # type: ignore[arg-type]
-                    new_speed=speed,
+                    new_rate=self._fluid_factory.mass_rate_to_standard_rate(mass_rate),  # type: ignore[arg-type]
                 )
             )
 
@@ -268,7 +262,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             """
             return self.evaluate_given_constraints(
                 constraints=constraints.create_conditions_with_new_input(
-                    new_rate=self.fluid_factory.mass_rate_to_standard_rate(mass_rate),  # type: ignore[arg-type]
+                    new_rate=self._fluid_factory.mass_rate_to_standard_rate(mass_rate),  # type: ignore[arg-type]
                 )
             )
 
@@ -279,8 +273,10 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
             Same as above, but mass rate is pinned to the "stone wall" as a function of speed.
             """
             _max_valid_mass_rate_at_given_speed = maximize_x_given_boolean_condition_function(
-                x_min=self.stages[0].compressor_chart.minimum_rate_as_function_of_speed(speed) * inlet_density,
-                x_max=self.stages[0].compressor_chart.maximum_rate_as_function_of_speed(speed) * inlet_density,
+                x_min=self.stages[0].compressor.compressor_chart.minimum_rate_as_function_of_speed(speed)
+                * inlet_density,
+                x_max=self.stages[0].compressor.compressor_chart.maximum_rate_as_function_of_speed(speed)
+                * inlet_density,
                 bool_func=lambda x: _calculate_train_result(mass_rate=x, speed=speed).within_capacity,
                 convergence_tolerance=1e-3,
                 maximum_number_of_iterations=20,
@@ -288,8 +284,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
 
             return self.calculate_compressor_train(
                 constraints=constraints.create_conditions_with_new_input(
-                    new_rate=self.fluid_factory.mass_rate_to_standard_rate(_max_valid_mass_rate_at_given_speed),  # type: ignore[arg-type]
-                    new_speed=speed,
+                    new_rate=self._fluid_factory.mass_rate_to_standard_rate(_max_valid_mass_rate_at_given_speed),  # type: ignore[arg-type]
                 )
             )
 
@@ -304,14 +299,14 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
 
         # Using first stage as absolute (initial) bounds on min and max rate at max speed. Checking validity later.
         min_mass_rate_at_max_speed_first_stage = (
-            self.stages[0].compressor_chart.maximum_speed_curve.minimum_rate * inlet_density
+            self.stages[0].compressor.compressor_chart.maximum_speed_curve.minimum_rate * inlet_density
         )
         max_mass_rate_at_max_speed_first_stage = (
-            self.stages[0].compressor_chart.maximum_speed_curve.maximum_rate * inlet_density
+            self.stages[0].compressor.compressor_chart.maximum_speed_curve.maximum_rate * inlet_density
         )
         max_mass_rate_at_min_speed_first_stage = (
-            self.stages[0].compressor_chart.maximum_rate_as_function_of_speed(
-                self.stages[0].compressor_chart.minimum_speed
+            self.stages[0].compressor.compressor_chart.maximum_rate_as_function_of_speed(
+                self.stages[0].compressor.compressor_chart.minimum_speed
             )
             * inlet_density
         )
@@ -471,12 +466,12 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         # If so, reduce rate such that power comes below maximum power
         maximum_power = self.maximum_power
         if not maximum_power:
-            result = self.fluid_factory.mass_rate_to_standard_rate(rate_to_return)
+            result = self._fluid_factory.mass_rate_to_standard_rate(rate_to_return)
             return float(result)
         elif (
             self.evaluate_given_constraints(
                 constraints=constraints.create_conditions_with_new_input(
-                    new_rate=self.fluid_factory.mass_rate_to_standard_rate(rate_to_return),  # type: ignore[arg-type]
+                    new_rate=self._fluid_factory.mass_rate_to_standard_rate(rate_to_return),  # type: ignore[arg-type]
                 )
             ).power_megawatt
             > maximum_power
@@ -493,13 +488,13 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
                 # iterate between rate with minimum power, and the previously found rate to return, to find the
                 # maximum rate that gives power consumption below maximum power
 
-                result = self.fluid_factory.mass_rate_to_standard_rate(
+                result = self._fluid_factory.mass_rate_to_standard_rate(
                     find_root(
                         lower_bound=result_with_minimum_rate.stage_results[0].mass_rate_asv_corrected_kg_per_hour,
                         upper_bound=rate_to_return,
                         func=lambda x: self.evaluate_given_constraints(
                             constraints=constraints.create_conditions_with_new_input(
-                                new_rate=self.fluid_factory.mass_rate_to_standard_rate(x),  # type: ignore[arg-type]
+                                new_rate=self._fluid_factory.mass_rate_to_standard_rate(x),  # type: ignore[arg-type]
                             )
                         ).power_megawatt
                         - maximum_power * (1 - POWER_CALCULATION_TOLERANCE),
@@ -510,7 +505,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
                 return float(result)
         else:
             # maximum power defined, but found rate is below maximum power
-            result = self.fluid_factory.mass_rate_to_standard_rate(rate_to_return)
+            result = self._fluid_factory.mass_rate_to_standard_rate(rate_to_return)
             return float(result)
 
     def _validate_maximum_discharge_pressure(self):
@@ -519,6 +514,20 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
 
             raise ProcessDischargePressureValidationException(message=str(msg))
 
+    def _validate_shaft(self):
+        if not isinstance(self.shaft, Shaft):
+            raise ValueError("A common shaft compressor train must have a shaft defined")
+        if self.is_variable_speed and isinstance(self.shaft, SingleSpeedShaft):
+            raise ValueError(
+                "A common shaft compressor train with multiple speeds must have a variable speed shaft defined."
+            )
+        elif not self.is_variable_speed:
+            if isinstance(self.shaft, VariableSpeedShaft):
+                raise ValueError(
+                    "A common shaft compressor train with a single speed must have a single speed shaft defined."
+                )
+            self.shaft.set_speed(self.minimum_speed)
+
     def evaluate_with_pressure_control_given_constraints(
         self, constraints: CompressorTrainEvaluationInput
     ) -> CompressorTrainResultSingleTimeStep:
@@ -530,6 +539,10 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         Returns:
             CompressorTrainResultSingleTimeStep: The result of the compressor train evaluation.
         """
+        if not self.shaft.speed_is_defined:
+            raise ValueError(
+                "Pressure control can only be applied after a speed curve is selected / the speed is fixed."
+            )
         if self.pressure_control == FixedSpeedPressureControl.DOWNSTREAM_CHOKE:
             train_result = self._evaluate_train_with_downstream_choking(
                 constraints=constraints,
@@ -622,10 +635,10 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         assert constraints.rate is not None
         assert constraints.suction_pressure is not None
 
-        train_inlet_stream = self.fluid_factory.create_stream_from_standard_rate(
-            pressure_bara=constraints.suction_pressure,
-            temperature_kelvin=self.stages[0].inlet_temperature_kelvin,
-            standard_rate_m3_per_day=constraints.rate,
+        train_inlet_stream = self.train_inlet_stream(
+            pressure=constraints.suction_pressure,
+            temperature=self.stages[0].inlet_temperature_kelvin,
+            rate=constraints.rate,
         )
 
         def _calculate_train_result_given_inlet_pressure(
@@ -643,7 +656,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         target_discharge_pressure = constraints.discharge_pressure
 
         result_inlet_pressure = find_root(
-            lower_bound=EPSILON + self.stages[0].pressure_drop_ahead_of_stage,  # type: ignore
+            lower_bound=EPSILON + self.stages[0].pressure_drop_ahead_of_stage,
             upper_bound=target_discharge_pressure,
             func=lambda x: _calculate_train_result_given_inlet_pressure(inlet_pressure=x).discharge_pressure
             - target_discharge_pressure,
@@ -742,7 +755,13 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         assert constraints.suction_pressure is not None
         assert constraints.discharge_pressure is not None
 
-        inlet_stream_train = self.fluid_factory.create_stream_from_standard_rate(
+        # Multiple streams factories are lists (one per stream).
+        if isinstance(self._fluid_factory, list):
+            fluid_factory = self._fluid_factory[0]
+        else:
+            fluid_factory = self._fluid_factory
+
+        inlet_stream_train = fluid_factory.create_stream_from_standard_rate(
             pressure_bara=constraints.suction_pressure,
             temperature_kelvin=self.stages[0].inlet_temperature_kelvin,
             standard_rate_m3_per_day=constraints.rate,  # type: ignore[arg-type]
@@ -756,7 +775,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         for stage in self.stages:
             outlet_pressure_for_stage = inlet_stream_stage.pressure_bara * pressure_ratio_per_stage
             stage_result = stage.evaluate_given_speed_and_target_discharge_pressure(
-                target_discharge_pressure=outlet_pressure_for_stage,  # type: ignore[arg-type]
+                target_discharge_pressure=outlet_pressure_for_stage,
                 inlet_stream_stage=inlet_stream_stage,
             )
             inlet_stream_stage = stage_result.outlet_stream
@@ -796,11 +815,16 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         Returns:
             CompressorTrainResultSingleTimeStep: The result of the evaluation for a single time step.
         """
-        minimum_mass_rate_kg_per_hour = self.fluid_factory.standard_rate_to_mass_rate(
+        # Multiple streams factories are lists (one per stream).
+        if isinstance(self._fluid_factory, list):
+            fluid_factory = self._fluid_factory[0]
+        else:
+            fluid_factory = self._fluid_factory
+        minimum_mass_rate_kg_per_hour = fluid_factory.standard_rate_to_mass_rate(
             standard_rate_m3_per_day=constraints.rate,  # type: ignore[arg-type]
         )
         # Iterate on rate until pressures are met
-        density_train_inlet_fluid = self.fluid_factory.create_thermo_system(
+        density_train_inlet_fluid = fluid_factory.create_thermo_system(
             pressure_bara=constraints.suction_pressure,  # type: ignore[arg-type]
             temperature_kelvin=self.stages[0].inlet_temperature_kelvin,
         ).density
@@ -810,7 +834,7 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         ) -> CompressorTrainResultSingleTimeStep:
             return self.calculate_compressor_train(
                 constraints=constraints.create_conditions_with_new_input(
-                    new_rate=self.fluid_factory.mass_rate_to_standard_rate(mass_rate_kg_per_h=mass_rate_kg_per_hour),  # type: ignore[arg-type]
+                    new_rate=fluid_factory.mass_rate_to_standard_rate(mass_rate_kg_per_h=mass_rate_kg_per_hour),  # type: ignore[arg-type]
                 ),
             )
 
@@ -826,10 +850,12 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         # minimum and maximum mass rate for the first stage, adjusted for the volume entering the first stage
         minimum_mass_rate = max(
             minimum_mass_rate_kg_per_hour,
-            self.stages[0].compressor_chart.minimum_rate * density_train_inlet_fluid,
+            self.stages[0].compressor.compressor_chart.minimum_rate * density_train_inlet_fluid,
         )  # type: ignore[type-var]
         # note: we subtract EPSILON to avoid floating point issues causing the maximum mass rate to exceed chart area maximum rate after round-trip conversion (mass rate -> standard rat -> mass rate)
-        maximum_mass_rate = self.stages[0].compressor_chart.maximum_rate * density_train_inlet_fluid * (1 - EPSILON)
+        maximum_mass_rate = (
+            self.stages[0].compressor.compressor_chart.maximum_rate * density_train_inlet_fluid * (1 - EPSILON)
+        )
 
         # if the minimum_mass_rate_kg_per_hour(i.e. before increasing rate with recirculation to lower pressure)
         # is already larger than the maximum mass rate, there is no need for optimization - just add result
@@ -940,3 +966,92 @@ class CompressorTrainCommonShaft(CompressorTrainModel):
         return _calculate_train_result_given_additional_mass_rate(
             additional_mass_rate_kg_per_hour=(result_mass_rate - minimum_mass_rate_kg_per_hour)  # type: ignore[arg-type]
         )
+
+    def find_fixed_shaft_speed_given_constraints(
+        self,
+        constraints: CompressorTrainEvaluationInput,
+        lower_bound_for_speed: float | None = None,
+        upper_bound_for_speed: float | None = None,
+    ) -> float:
+        """Calculate needed shaft speed to get desired outlet pressure
+
+        Run compressor train forward model with inlet conditions and speed, and iterate on shaft speed until discharge
+        pressure meets requested discharge pressure.
+
+        Iteration (using brenth method) to find speed to meet requested discharge pressure
+
+        The upper and lower bounds for the speed can be set, which is useful for a part of a compressor train that can
+        share a common shaft with another part of a compressor train, which has another minimum and maximum speed.
+
+        Iterative problem:
+            f(speed) = calculate_compressor_train(speed).discharge_pressure - requested_discharge_pressure = 0
+        Starting points for iterative method:
+           speed_0 = minimum speed for train, calculate f(speed_0) aka f_0
+           speed_1 = maximum speed for train, calculate f(speed_1) aka f_1
+
+        Args:
+            constraints (CompressorTrainEvaluationInput): The constraints for the evaluation.
+            lower_bound_for_speed (float | None): The lower bound for the speed. If None, uses the minimum speed
+            upper_bound_for_speed (float | None): The upper bound for the speed. If None, uses the maximum speed
+        Returns:
+            The speed required to operate at to meet the given constraints. (Bounded by the minimu and maximum speed)
+
+        """
+        minimum_speed = (
+            lower_bound_for_speed
+            if lower_bound_for_speed and lower_bound_for_speed > self.minimum_speed
+            else self.minimum_speed
+        )
+        maximum_speed = (
+            upper_bound_for_speed
+            if upper_bound_for_speed and upper_bound_for_speed < self.maximum_speed
+            else self.maximum_speed
+        )
+
+        def _calculate_compressor_train(_speed: float) -> CompressorTrainResultSingleTimeStep:
+            self.shaft.set_speed(_speed)
+            return self.calculate_compressor_train(
+                constraints=constraints,
+            )
+
+        train_result_for_minimum_speed = _calculate_compressor_train(_speed=minimum_speed)
+        train_result_for_maximum_speed = _calculate_compressor_train(_speed=maximum_speed)
+
+        if not train_result_for_maximum_speed.within_capacity:
+            # will not find valid result - the rate is above maximum rate, return invalid results at maximum speed
+            return maximum_speed
+        if not train_result_for_minimum_speed.within_capacity:
+            # rate is above maximum rate for minimum speed. Find the lowest minimum speed which gives a valid result
+            minimum_speed = -maximize_x_given_boolean_condition_function(
+                x_min=-maximum_speed,
+                x_max=-minimum_speed,
+                bool_func=lambda x: _calculate_compressor_train(_speed=-x).within_capacity,
+            )
+            train_result_for_minimum_speed = _calculate_compressor_train(_speed=minimum_speed)
+
+        # Solution 1, iterate on speed until target discharge pressure is found
+        if (
+            constraints.discharge_pressure is not None
+            and train_result_for_minimum_speed.discharge_pressure
+            <= constraints.discharge_pressure
+            <= train_result_for_maximum_speed.discharge_pressure
+        ):
+            # At this point, discharge_pressure is confirmed to be not None
+            target_discharge_pressure = constraints.discharge_pressure
+            speed = find_root(
+                lower_bound=minimum_speed,
+                upper_bound=maximum_speed,
+                func=lambda x: _calculate_compressor_train(_speed=x).discharge_pressure - target_discharge_pressure,
+            )
+
+            return speed
+
+        # Solution 2, target pressure is too low:
+        if (
+            constraints.discharge_pressure is not None
+            and constraints.discharge_pressure < train_result_for_minimum_speed.discharge_pressure
+        ):
+            return minimum_speed
+
+        # Solution 3, target discharge pressure is too high
+        return maximum_speed
