@@ -7,7 +7,6 @@ from numpy._typing import NDArray
 from libecalc.common.errors.exceptions import IllegalStateException
 from libecalc.common.fixed_speed_pressure_control import FixedSpeedPressureControl
 from libecalc.common.logger import logger
-from libecalc.common.serializable_chart import ChartDTO
 from libecalc.domain.component_validation_error import ProcessChartTypeValidationException
 from libecalc.domain.process.compressor.core.results import CompressorTrainResultSingleTimeStep
 from libecalc.domain.process.compressor.core.train.compressor_train_common_shaft import CompressorTrainCommonShaft
@@ -19,8 +18,8 @@ from libecalc.domain.process.compressor.core.train.utils.numeric_methods import
     maximize_x_given_boolean_condition_function,
 )
 from libecalc.domain.process.core.results.compressor import TargetPressureStatus
-from libecalc.domain.process.value_objects.chart.compressor import CompressorChart
-from libecalc.domain.process.value_objects.fluid_stream import ProcessConditions, SimplifiedStreamMixing
+from libecalc.domain.process.entities.shaft import Shaft, VariableSpeedShaft
+from libecalc.domain.process.value_objects.fluid_stream import FluidStream, ProcessConditions, SimplifiedStreamMixing
 from libecalc.domain.process.value_objects.fluid_stream.fluid_factory import FluidFactoryInterface
 from libecalc.domain.process.value_objects.fluid_stream.fluid_model import FluidModel
 
@@ -59,10 +58,10 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
     def __init__(
         self,
         streams: list[FluidStreamObjectForMultipleStreams],
-        fluid_factory: FluidFactoryInterface,
         energy_usage_adjustment_constant: float,
         energy_usage_adjustment_factor: float,
         stages: list[CompressorTrainStage],
+        shaft: Shaft,
         calculate_max_rate: bool = False,
         maximum_power: float | None = None,
         pressure_control: FixedSpeedPressureControl | None = None,
@@ -70,10 +69,10 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
     ):
         logger.debug(f"Creating {type(self).__name__} with\n" f"n_stages: {len(stages)} and n_streams: {len(streams)}")
         super().__init__(
-            fluid_factory=fluid_factory,
             energy_usage_adjustment_constant=energy_usage_adjustment_constant,
             energy_usage_adjustment_factor=energy_usage_adjustment_factor,
             stages=stages,
+            shaft=shaft,
             maximum_power=maximum_power,
             pressure_control=pressure_control,
             calculate_max_rate=calculate_max_rate,
@@ -85,16 +84,6 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
         self.streams = streams
         self.number_of_compressor_streams = len(self.streams)
 
-        # Create fluid factories for each stream
-        self.stream_fluid_factories: list[FluidFactoryInterface | None] = []
-        for stream in self.streams:
-            if stream.fluid_model is not None:
-                self.stream_fluid_factories.append(
-                    fluid_factory.create_fluid_factory_from_fluid_model(stream.fluid_model)
-                )
-            else:
-                self.stream_fluid_factories.append(None)
-
         self.inlet_stream_connected_to_stage: dict[int, list[int]] = {key: [] for key in range(len(self.stages))}
         self.outlet_stream_connected_to_stage: dict[int, list[int]] = {key: [] for key in range(len(self.stages))}
         for i, stream in enumerate(self.streams):
@@ -122,6 +111,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
     def set_evaluation_input(
         self,
         rate: NDArray[np.float64],
+        fluid_factory: FluidFactoryInterface | list[FluidFactoryInterface] | None,
         suction_pressure: NDArray[np.float64] | None,
         discharge_pressure: NDArray[np.float64] | None,
         intermediate_pressure: NDArray[np.float64] | None = None,
@@ -136,6 +126,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             suction_pressure=suction_pressure,
             discharge_pressure=discharge_pressure,
             intermediate_pressure=intermediate_pressure,
+            fluid_factory=fluid_factory,
         )
 
     @staticmethod
@@ -169,21 +160,13 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             logger.exception(msg)
             raise IllegalStateException(msg)
 
-    def _validate_stages(self, stages):
+    def _validate_stages(self, stages: list[CompressorTrainStage]):
         min_speed_per_stage = []
         max_speed_per_stage = []
         for stage in stages:
-            if not isinstance(stage.compressor_chart, CompressorChart | ChartDTO):
-                msg = "Variable Speed Compressor train only accepts Variable Speed Compressor Charts."
-                f" Given type was {type(stage.compressor_chart)}"
+            max_speed_per_stage.append(stage.compressor.compressor_chart.maximum_speed)
+            min_speed_per_stage.append(stage.compressor.compressor_chart.minimum_speed)
 
-                raise ProcessChartTypeValidationException(message=str(msg))
-            if isinstance(stage.compressor_chart, CompressorChart):
-                max_speed_per_stage.append(stage.compressor_chart.maximum_speed)
-                min_speed_per_stage.append(stage.compressor_chart.minimum_speed)
-            else:
-                max_speed_per_stage.append(stage.compressor_chart.max_speed)
-                min_speed_per_stage.append(stage.compressor_chart.min_speed)
         if max(min_speed_per_stage) > min(max_speed_per_stage):
             msg = "Variable speed compressors in compressor train have incompatible compressor charts."
             f" Stage {min_speed_per_stage.index(max(min_speed_per_stage)) + 1}'s minimum speed is higher"
@@ -191,9 +174,33 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
 
             raise ProcessChartTypeValidationException(message=str(msg))
 
+    def train_inlet_stream(
+        self,
+        pressure: float,
+        temperature: float,
+        rate: float,
+    ) -> FluidStream:
+        """Find inlet stream given constraints.
+
+        Args:
+            pressure (float):
+            temperature (float):
+            rate (float):
+
+        Returns:
+            FluidStream: Inlet fluid stream at the compressor train inlet.
+        """
+        assert isinstance(self._fluid_factory, list)  # for mypy
+        return self._fluid_factory[0].create_stream_from_standard_rate(
+            pressure_bara=pressure,
+            temperature_kelvin=temperature,
+            standard_rate_m3_per_day=rate,
+        )
+
     def evaluate_given_constraints(
         self,
         constraints: CompressorTrainEvaluationInput,
+        fixed_speed: float | None = None,
     ) -> CompressorTrainResultSingleTimeStep:
         """
         Evaluate the compressor train for the given operating constraints.
@@ -205,7 +212,9 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
 
         Args:
             constraints (CompressorTrainEvaluationInput): The operating constraints, including pressures, stream rates,
-                and optional speed or intermediate pressure.
+                and optional intermediate pressure.
+            fixed_speed (float): A fixed constant speed for the compressor train. Takes away the ability to iterate
+                over speed to find a solution. Defaults to None.
 
         Returns:
             CompressorTrainResultSingleTimeStep: The result of the simulation, including stage results, inlet/outlet streams,
@@ -229,6 +238,8 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             return CompressorTrainResultSingleTimeStep.create_empty(number_of_stages=len(self.stages))
         else:
             if constraints.interstage_pressure is not None:
+                if fixed_speed is not None:
+                    raise ValueError("You can not set a fixed speed when also setting an interstage pressure.")
                 self._check_intermediate_pressure_stage_number_is_valid(
                     _stage_number_intermediate_pressure=self.stage_number_interstage_pressure,
                     number_of_stages=len(self.stages),
@@ -240,20 +251,12 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
                     pressure_control_last_part=self.pressure_control_last_part,
                 )
             else:
-                if constraints.speed is None:
-                    speed = self.find_shaft_speed_given_constraints(
-                        constraints=constraints,
-                    )
-                    train_result = self.calculate_compressor_train(
-                        constraints=constraints.create_conditions_with_new_input(
-                            new_speed=speed,
-                        ),
-                    )
-                else:
-                    speed = constraints.speed
-                    train_result = self.calculate_compressor_train(
-                        constraints=constraints,
-                    )
+                if fixed_speed is None:
+                    fixed_speed = self.find_fixed_shaft_speed_given_constraints(constraints=constraints)
+                self.shaft.set_speed(fixed_speed)
+                train_result = self.calculate_compressor_train(
+                    constraints=constraints,
+                )
 
                 if train_result.target_pressure_status == TargetPressureStatus.TARGET_PRESSURES_MET:
                     # Solution found
@@ -266,9 +269,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
                     return train_result
                 else:
                     train_result = self.evaluate_with_pressure_control_given_constraints(
-                        constraints=constraints.create_conditions_with_new_input(
-                            new_speed=speed,
-                        )
+                        constraints=constraints,
                     )
                 return train_result
 
@@ -397,11 +398,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             )
 
         train_result = self.evaluate_given_constraints(
-            constraints=constraints.create_conditions_with_new_input(
-                new_speed=self.find_shaft_speed_given_constraints(
-                    constraints=constraints,
-                )
-            )
+            constraints=constraints,
         )
         if not train_result.is_valid:
             zero_stream_result = _calculate_train_result(std_rate_for_stream=EPSILON)
@@ -422,12 +419,6 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             std_rates_std_m3_per_day_per_stream[constraints.stream_to_maximize] = max_rate_is_larger_than * 2
             train_result = self.evaluate_given_constraints(
                 constraints=constraints.create_conditions_with_new_input(
-                    new_speed=self.find_shaft_speed_given_constraints(
-                        constraints=constraints.create_conditions_with_new_input(
-                            new_rate=std_rates_std_m3_per_day_per_stream[0],
-                            new_stream_rates=std_rates_std_m3_per_day_per_stream,
-                        ),
-                    ),
                     new_rate=std_rates_std_m3_per_day_per_stream[0],
                     new_stream_rates=std_rates_std_m3_per_day_per_stream,
                 )
@@ -469,16 +460,16 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
         # This multiple streams train also requires stream_rates to be set
         assert constraints.stream_rates is not None
         assert constraints.suction_pressure is not None
-        assert constraints.speed is not None
         mixing_strategy = SimplifiedStreamMixing()
         stage_results = []
         # Make list of fluid streams for the ingoing streams
+        assert isinstance(self._fluid_factory, list)  # for mypy
         fluid_streams = []
         for i, stream in enumerate(self.streams):
             if stream.is_inlet_stream:
-                if self.stream_fluid_factories[i] is not None:
+                if self._fluid_factory[i] is not None:
                     fluid_streams.append(
-                        self.stream_fluid_factories[i].create_stream_from_standard_rate(
+                        self._fluid_factory[i].create_stream_from_standard_rate(
                             pressure_bara=constraints.suction_pressure,
                             temperature_kelvin=self.stages[0].inlet_temperature_kelvin,
                             standard_rate_m3_per_day=constraints.stream_rates[i],
@@ -522,7 +513,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             stage_results.append(
                 stage.evaluate(
                     inlet_stream_stage=stage_inlet_stream,
-                    speed=constraints.speed,
+                    speed=self.shaft.get_speed(),
                     asv_rate_fraction=asv_rate_fraction,
                     asv_additional_mass_rate=asv_additional_mass_rate,
                 )
@@ -540,7 +531,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             inlet_stream=train_inlet_stream,
             outlet_stream=previous_stage_outlet_stream,
             stage_results=stage_results,
-            speed=constraints.speed or float("nan"),
+            speed=self.shaft.get_speed(),
             above_maximum_power=sum([stage_result.power_megawatt for stage_result in stage_results])
             > self.maximum_power
             if self.maximum_power
@@ -606,44 +597,24 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             stream_rates=std_rates_last_part,
         )
 
-        compressor_train_first_part_optimal_speed = compressor_train_first_part.find_shaft_speed_given_constraints(
+        compressor_train_first_part.find_fixed_shaft_speed_given_constraints(
             constraints=constraints_first_part,
             lower_bound_for_speed=self.minimum_speed,  # Only search for a solution within the bounds of the
             upper_bound_for_speed=self.maximum_speed,  # original, complete compressor train
         )
-        if math.isclose(compressor_train_first_part_optimal_speed, self.minimum_speed, rel_tol=EPSILON):
+        if math.isclose(compressor_train_first_part.shaft.get_speed(), self.minimum_speed, rel_tol=EPSILON):
             compressor_train_results_first_part_with_optimal_speed_result = (
                 compressor_train_first_part.evaluate_with_pressure_control_given_constraints(
-                    constraints=constraints_first_part.create_conditions_with_new_input(
-                        new_speed=compressor_train_first_part_optimal_speed,
-                    )
+                    constraints=constraints_first_part,
                 )
             )
         else:
             compressor_train_results_first_part_with_optimal_speed_result = (
                 compressor_train_first_part.calculate_compressor_train(
-                    constraints=constraints_first_part.create_conditions_with_new_input(
-                        new_speed=compressor_train_first_part_optimal_speed,
-                    )
+                    constraints=constraints_first_part,
                 )
             )
 
-        if self.calculate_max_rate:
-            max_standard_rate_per_stream = [
-                compressor_train_first_part.get_max_standard_rate(  # type: ignore[call-arg]
-                    constraints=CompressorTrainEvaluationInput(
-                        rate=std_rates_first_part[0],
-                        suction_pressure=constraints_first_part.suction_pressure,
-                        discharge_pressure=constraints_first_part.discharge_pressure,
-                        stream_rates=std_rates_first_part,
-                        stream_to_maximize=stream_index,
-                    ),
-                )
-                for stream_index, _ in enumerate(compressor_train_first_part.streams)
-            ]
-        else:
-            max_standard_rate_per_stream = [float("nan")] * len(std_rates_first_part)  # type: ignore[list-item]
-
         # set self.inlet_fluid based on outlet_stream_first_part
         compressor_train_last_part.streams[0].fluid_model = FluidModel(
             composition=compressor_train_results_first_part_with_optimal_speed_result.stage_results[
@@ -656,65 +627,40 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
 
         # Update fluid factory to match the new fluid model
         # This ensures base class methods use the correct fluid properties/composition
-        compressor_train_last_part.fluid_factory = (
-            compressor_train_last_part.fluid_factory.create_fluid_factory_from_fluid_model(
-                compressor_train_last_part.streams[0].fluid_model
-            )
-        )
-        # Also update the stream_fluid_factories for consistency
-        compressor_train_last_part.stream_fluid_factories[0] = compressor_train_last_part.fluid_factory
+        assert isinstance(compressor_train_last_part._fluid_factory, list)  # for mypy
+        compressor_train_last_part._fluid_factory[0] = compressor_train_last_part._fluid_factory[
+            0
+        ].create_fluid_factory_from_fluid_model(compressor_train_last_part.streams[0].fluid_model)
 
-        compressor_train_last_part_optimal_speed = compressor_train_last_part.find_shaft_speed_given_constraints(
+        compressor_train_last_part.find_fixed_shaft_speed_given_constraints(
             constraints=constraints_last_part,
             lower_bound_for_speed=self.minimum_speed,
             upper_bound_for_speed=self.maximum_speed,
         )
 
-        if math.isclose(compressor_train_last_part_optimal_speed, self.minimum_speed, rel_tol=EPSILON):
+        if math.isclose(compressor_train_last_part.shaft.get_speed(), self.minimum_speed, rel_tol=EPSILON):
             compressor_train_results_last_part_with_optimal_speed_result = (
                 compressor_train_last_part.evaluate_with_pressure_control_given_constraints(
-                    constraints=constraints_last_part.create_conditions_with_new_input(
-                        new_speed=compressor_train_last_part_optimal_speed,
-                    )
+                    constraints=constraints_last_part,
                 )
             )
         else:
             compressor_train_results_last_part_with_optimal_speed_result = (
                 compressor_train_last_part.calculate_compressor_train(
-                    constraints=constraints_last_part.create_conditions_with_new_input(
-                        new_speed=compressor_train_last_part_optimal_speed,
-                    )
+                    constraints=constraints_last_part,
                 )
             )
 
-        if self.calculate_max_rate:
-            for stream_index, _ in enumerate(compressor_train_last_part.streams):
-                if stream_index > 0:
-                    max_standard_rate_per_stream.append(
-                        compressor_train_first_part.get_max_standard_rate(  # type: ignore[call-arg]
-                            constraints=CompressorTrainEvaluationInput(
-                                rate=std_rates_last_part[0],
-                                suction_pressure=constraints_last_part.suction_pressure,
-                                discharge_pressure=constraints_last_part.discharge_pressure,
-                                stream_rates=std_rates_last_part,
-                                stream_to_maximize=stream_index,
-                            ),
-                        )
-                    )
-        else:
-            max_standard_rate_per_stream = max_standard_rate_per_stream + [float("nan")] * len(std_rates_last_part[1:])  # type: ignore[list-item]
         """
         Determine which of the first and last part will govern the speed to use
         Then run the last part as a single speed train with the speed chosen
         Fixme: Need to deliver the result in a proper format below.
         """
-        if compressor_train_first_part_optimal_speed > compressor_train_last_part_optimal_speed:
-            speed = compressor_train_first_part_optimal_speed
+        if compressor_train_first_part.shaft.get_speed() > compressor_train_last_part.shaft.get_speed():
+            compressor_train_last_part.shaft.set_speed(compressor_train_first_part.shaft.get_speed())
             compressor_train_results_last_part_with_pressure_control = (
                 compressor_train_last_part.evaluate_with_pressure_control_given_constraints(
-                    constraints=constraints_last_part.create_conditions_with_new_input(
-                        new_speed=speed,
-                    )
+                    constraints=constraints_last_part,
                 )
             )
             compressor_train_results_to_return_first_part = (
@@ -723,12 +669,10 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
             compressor_train_results_to_return_last_part = compressor_train_results_last_part_with_pressure_control
 
         else:
-            speed = compressor_train_last_part_optimal_speed
+            compressor_train_first_part.shaft.set_speed(compressor_train_last_part.shaft.get_speed())
             compressor_train_results_first_part_with_pressure_control = (
                 compressor_train_first_part.evaluate_with_pressure_control_given_constraints(
-                    constraints=constraints_first_part.create_conditions_with_new_input(
-                        new_speed=speed,
-                    )
+                    constraints=constraints_first_part,
                 )
             )
             compressor_train_results_to_return_first_part = compressor_train_results_first_part_with_pressure_control
@@ -742,7 +686,7 @@ class CompressorTrainCommonShaftMultipleStreamsAndPressures(CompressorTrainCommo
         train_result = CompressorTrainResultSingleTimeStep(
             inlet_stream=compressor_train_results_to_return_first_part.inlet_stream,
             outlet_stream=compressor_train_results_to_return_last_part.outlet_stream,
-            speed=speed,
+            speed=compressor_train_first_part.shaft.get_speed(),
             stage_results=compressor_train_results_to_return_stage_results,
             above_maximum_power=sum(
                 [stage_result.power_megawatt for stage_result in compressor_train_results_to_return_stage_results]
@@ -831,13 +775,16 @@ def split_train_on_stage_number(
 
     # Create streams for first part
     streams_first_part = [stream for stream in compressor_train.streams if stream.connected_to_stage_no < stage_number]
-
-    # First part uses the main fluid factory (already made from first stream)
-    fluid_factory_first_part = compressor_train.fluid_factory
+    assert isinstance(compressor_train._fluid_factory, list)  # for mypy
+    fluid_factory_first_part = [
+        fluid_factory
+        for stream, fluid_factory in zip(compressor_train.streams, compressor_train._fluid_factory)
+        if stream.connected_to_stage_no < stage_number
+    ]
 
     compressor_train_first_part = CompressorTrainCommonShaftMultipleStreamsAndPressures(
         streams=streams_first_part,
-        fluid_factory=fluid_factory_first_part,
+        shaft=VariableSpeedShaft(),
         energy_usage_adjustment_constant=compressor_train.energy_usage_adjustment_constant,
         energy_usage_adjustment_factor=compressor_train.energy_usage_adjustment_factor,
         stages=compressor_train.stages[:stage_number],
@@ -849,6 +796,8 @@ def split_train_on_stage_number(
         stage_number_interstage_pressure=compressor_train.stage_number_interstage_pressure,
     )
 
+    compressor_train_first_part._fluid_factory = fluid_factory_first_part
+
     # Create streams for last part
     streams_last_part = [
         deepcopy(compressor_train.streams[compressor_train.inlet_stream_connected_to_stage.get(0)[0]])  # type: ignore[index]
@@ -864,17 +813,24 @@ def split_train_on_stage_number(
             if stream.connected_to_stage_no >= stage_number
         ]
     )
-
     # Last part initially uses the main fluid factory (placeholder)
     # This will be updated at runtime after the fluid model (composition) is changed
-    fluid_factory_last_part = compressor_train.fluid_factory
+
+    fluid_factory_last_part = [deepcopy(compressor_train._fluid_factory[0])]
+    fluid_factory_last_part.extend(
+        [
+            fluid_factory
+            for stream, fluid_factory in zip(compressor_train.streams, compressor_train._fluid_factory)
+            if stream.connected_to_stage_no >= stage_number
+        ]
+    )
 
     compressor_train_last_part = CompressorTrainCommonShaftMultipleStreamsAndPressures(
         streams=streams_last_part,
-        fluid_factory=fluid_factory_last_part,
         energy_usage_adjustment_constant=compressor_train.energy_usage_adjustment_constant,
         energy_usage_adjustment_factor=compressor_train.energy_usage_adjustment_factor,
         stages=compressor_train.stages[stage_number:],
+        shaft=VariableSpeedShaft(),
         calculate_max_rate=compressor_train.calculate_max_rate
         if compressor_train.calculate_max_rate is not None
         else False,
@@ -882,5 +838,6 @@ def split_train_on_stage_number(
         pressure_control=pressure_control_last_part,
         stage_number_interstage_pressure=compressor_train.stage_number_interstage_pressure,
     )
+    compressor_train_last_part._fluid_factory = fluid_factory_last_part
 
     return compressor_train_first_part, compressor_train_last_part
