@@ -4,9 +4,9 @@ import abc
 from typing import Protocol
 from uuid import UUID
 
-from libecalc.common.serializable_chart import ChartDTO
 from libecalc.common.time_utils import Period
 from libecalc.common.units import Unit
+from libecalc.domain.process.value_objects.chart import Chart
 
 ProcessEntityID = UUID
 
@@ -77,4 +77,4 @@ class CompressorStage(abc.ABC):
     @abc.abstractmethod
     def get_compressor_chart(
         self,
-    ) -> ChartDTO | None: ...
+    ) -> Chart | None: ...
