@@ -1,15 +1,27 @@
+import abc
 from collections.abc import Callable
+from functools import cached_property
 from typing import Any
 
 import numpy as np
 from numpy.typing import NDArray
 from scipy.interpolate import interp1d
 
+from libecalc.common.chart_type import ChartType
 from libecalc.common.list.list_utils import array_to_list
-from libecalc.common.serializable_chart import ChartDTO
+from libecalc.domain.component_validation_error import ProcessChartTypeValidationException
 from libecalc.domain.process.value_objects.chart.base import ChartCurve
 
 
+class ChartData(abc.ABC):
+    @abc.abstractmethod
+    def get_curves(self) -> list[ChartCurve]: ...
+
+    @property
+    @abc.abstractmethod
+    def origin_of_chart_data(self) -> ChartType: ...
+
+
 class Chart:
     """When talking about charts, it is important to understand the different components of the chart:
 
@@ -69,9 +81,51 @@ class Chart:
 
     """
 
-    def __init__(self, data_transfer_object: ChartDTO):
-        self.data_transfer_object = data_transfer_object
-        self.curves = [ChartCurve(curve) for curve in data_transfer_object.curves]
+    def __init__(self, chart_data: ChartData):
+        """
+        TODO: Should we allow to instantiate a Chart directly? Or should we only allow instantiation through factory methods. Direct instantiation currently only from Pump...
+        Args:
+            chart_data:
+        """
+        self._chart_data = chart_data
+        self.__validate()
+
+    def __validate(self):
+        self.__check_that_there_is_at_least_one_chart_curve()
+
+    def __check_that_there_is_at_least_one_chart_curve(self):
+        if self._chart_data.origin_of_chart_data is ChartType.GENERIC_FROM_INPUT:
+            # Skip validation of number of curves. It will have two curves.
+            # Calling self.curves will require NeqSim again, which we want to avoid...
+            return
+        if len(self.curves) == 0:
+            msg = "At least one chart curve must be given to define a compressor performance chart."
+            raise ProcessChartTypeValidationException(message=msg)
+
+    @property
+    def chart_data(self) -> ChartData:
+        return self._chart_data
+
+    @property
+    def origin_of_chart_data(self) -> ChartType:
+        return self._chart_data.origin_of_chart_data
+
+    @cached_property
+    def curves(self) -> list[ChartCurve]:
+        if self.origin_of_chart_data in (ChartType.GENERIC_FROM_INPUT, ChartType.GENERIC_FROM_DESIGN_POINT):
+            # Skip getting adjusted curves. It will have two curves which are the same as original curves.
+            return self.original_curves
+        curves = self._chart_data.get_adjusted_curves()
+        return sorted(curves, key=lambda x: x.speed)
+
+    @cached_property
+    def original_curves(self) -> list[ChartCurve]:
+        curves = self._chart_data.get_curves()
+        return sorted(curves, key=lambda x: x.speed)
+
+    @property
+    def is_variable_speed(self) -> bool:
+        return len(self.curves) > 1
 
     @property
     def speed_values(self) -> list[float]:
@@ -364,14 +418,15 @@ class Chart:
 
         scaled_chart_curves = [
             # Note: the values will contain negative values which will result in a validation error unless we do this:
-            curve.data_transfer_object.model_copy(
-                update={
-                    "rate_actual_m3_hour": array_to_list((curve.rate_values - mean_rates) / std_rates),
-                    "polytropic_head_joule_per_kg": array_to_list((curve.head_values - mean_heads) / std_heads),
-                }
+            ChartCurve(
+                speed_rpm=curve.speed,
+                rate_actual_m3_hour=array_to_list((curve.rate_values - mean_rates) / std_rates),
+                polytropic_head_joule_per_kg=array_to_list((curve.head_values - mean_heads) / std_heads),
+                efficiency_fraction=curve.efficiency,
             )
             for curve in self.curves
         ]
+
         for i, rate in enumerate(scaled_rates):
             q = rate
             h = scaled_heads[i]
@@ -383,7 +438,10 @@ class Chart:
 
             for scaled_chart_curve in scaled_chart_curves:
                 distance, efficiency = ChartCurve(
-                    scaled_chart_curve
+                    rate_actual_m3_hour=scaled_chart_curve.rate_actual_m3_hour,
+                    polytropic_head_joule_per_kg=scaled_chart_curve.polytropic_head_joule_per_kg,
+                    efficiency_fraction=scaled_chart_curve.efficiency_fraction,
+                    speed_rpm=scaled_chart_curve.speed_rpm,
                 ).get_distance_and_efficiency_from_closest_point_on_curve(rate=q, head=h)
 
                 if 0 <= distance < distance_above:  # Curve is above this point
