@@ -12,12 +12,12 @@ from libecalc.presentation.yaml.mappers.yaml_path import YamlPath
 from libecalc.presentation.yaml.yaml_models.yaml_model import YamlValidator
 from libecalc.presentation.yaml.yaml_types.facility_model.yaml_facility_model import (
     YamlFacilityModel,
-    YamlFacilityModelType,
     YamlGeneratorSetModel,
     YamlPumpChartSingleSpeed,
     YamlPumpChartVariableSpeed,
     YamlTabularModel,
 )
+from libecalc.presentation.yaml.yaml_types.facility_model.yaml_facility_model_type import YamlFacilityModelType
 from libecalc.presentation.yaml.yaml_types.fuel_type.yaml_fuel_type import YamlFuelType
 from libecalc.presentation.yaml.yaml_types.models import (
     YamlCompressorChart,
